@@ -15,7 +15,7 @@
 
 #include <stddef.h>
 #include <X11/Xlib.h>
-#include <LTKWidget.h>
+#include "LTKWidget.h"
 
 //------------------------------------------------------------------------
 // LTKCanvas
@@ -24,13 +24,11 @@
 class LTKCanvas: public LTKWidget {
 public:
 
-  //---------- constructors and destructor ----------
+  //---------- constructor ----------
 
   LTKCanvas(char *name1, int widgetNum1,
 	    int minWidth1, int minHeight1, LTKRedrawCbk redrawCbk1);
 
-  virtual LTKWidget *copy() { return new LTKCanvas(this); }
-
   //---------- layout ----------
 
   virtual void layout1();
@@ -41,8 +39,6 @@ public:
 
 protected:
 
-  LTKCanvas(LTKCanvas *canvas);
-
   int minWidth, minHeight;	// minimum size
 
   LTKRedrawCbk redrawCbk;	// redraw callback
