@@ -16,12 +16,12 @@
 #include <X11/Xlib.h>
 #include <X11/Xresource.h>
 #include <X11/Xutil.h>
-#include <gtypes.h>
-#include <GString.h>
-#include <fileNames.h>
-#include <LTKConfig.h>
-#include <LTKMisc.h>
-#include <LTKResources.h>
+#include "gtypes.h"
+#include "GString.h"
+#include "gfile.h"
+#include "LTKConfig.h"
+#include "LTKMisc.h"
+#include "LTKResources.h"
 
 void ltkGetCmdLineResources(XrmDatabase *cmdLineDB, XrmOptionDescRec *opts,
 			    int numOpts, GString *appName,
@@ -103,6 +103,29 @@ int ltkGetIntResource(XrmDatabase db, GString *appName,
   return ret;
 }
 
+GBool ltkGetBoolResource(XrmDatabase db, GString *appName,
+			 char *instName, GBool def) {
+  GString *inst = appName->copy()->append(".")->append(instName);
+  XrmValue val;
+  char *resType[20];
+  char *s;
+  GBool ret;
+
+  if (XrmGetResource(db, inst->getCString(), inst->getCString(),
+		     resType, &val)) {
+    s = (char *)val.addr;
+    ret =  ((s[0] == 'o' || s[0] == 'O') && (s[1] == 'n' || s[1] == 'N')) ||
+	   ((s[0] == 't' || s[0] == 'T') && (s[1] == 'r' || s[1] == 'R') &&
+	    (s[2] == 'u' || s[2] == 'U') && (s[3] == 'e' || s[3] == 'E')) ||
+	   ((s[0] == 'y' || s[0] == 'Y') && (s[1] == 'e' || s[1] == 'E') &&
+	    (s[2] == 's' || s[2] == 'S'));
+  } else {
+    ret = def;
+  }
+  delete inst;
+  return ret;
+}
+
 unsigned long ltkGetColorResource(XrmDatabase db, GString *appName,
 				  char *instName,
 				  Display *display, int screenNum,
