@@ -9,15 +9,15 @@
 #ifndef PARAMS_H
 #define PARAMS_H
 
-// Size of RGB color cube.
-extern int rgbCubeSize;
-
 // Print commands as they're executed.
 extern GBool printCommands;
 
 // Send error messages to /dev/tty instead of stderr.
 extern GBool errorsToTTY;
 
+// If this is set, error messages will be silently discarded.
+extern GBool errQuiet;
+
 // Font search path.
 extern char **fontPath;
 
