@@ -14,7 +14,7 @@
 #include <stddef.h>
 #include <string.h>
 #include <ctype.h>
-#include <gmem.h>
+#include "gmem.h"
 #include "Object.h"
 #include "Stream.h"
 #include "Lexer.h"
@@ -437,3 +437,7 @@ Object *XRef::fetch(int num, int gen, Object *obj) {
   }
   return obj;
 }
+
+Object *XRef::getDocInfo(Object *obj) {
+  return trailerDict.dictLookup("Info", obj);
+}
